/*
 * SmartDoc : Ultimate document format based on XML
 *  Copyright (C) 1998-2002  ASAMI, Tomoharu (asami@XMLSmartDoc.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package jp.gr.java_conf.jaba2.SmartDoc;

import java.util.*;
import org.w3c.dom.*;
import jp.gr.java_conf.jaba2.text.UString;

/**
 * Article
 *
 * @since   Nov.  3, 1998
 * @version Jan. 29, 2002
 * @author  ASAMI, Tomoharu (asami@XMLSmartDoc.org)
 */
public class Article extends Bibitem {
    protected List authors_ = new ArrayList();
    protected String subtitle_;
    protected String pages_;
    protected String uri_;
    protected Journal journal_;

    public Article(Element element) {
	super(element);
	_init(element);
    }

    public Article(Element element, BibliographyDatabase bibDB) {
	super(element);
	String idref = getIdref();
	if (UString.notNull(idref)) {
	    Bibitem ref = bibDB.get(idref);
	    if (ref != null) {
		if (ref instanceof Article) {
		    Article refArticle = (Article)ref;
		    UDoc.copyAttr(refArticle, this);
		    authors_.addAll(refArticle.authors_);
		    subtitle_ = refArticle.subtitle_;
		    journal_ = refArticle.journal_;
		} else {
		    throw (new InternalError()); // XXX
		}
	    } else {
		_warning("no article = " + idref);
	    }
	}
	_init(element);
    }

    private void _init(Element element) {
	NodeList nodes = element.getChildNodes();
	int size = nodes.getLength();
	for (int i = 0;i < size;i++) {
	    Node node = nodes.item(i);
	    switch (node.getNodeType()) {

	    case Node.ELEMENT_NODE:
		Element child = (Element)node;
		String tagName = child.getTagName();
		if ("author".equals(tagName)) {
		    authors_.add(USmartDoc.getString(child));
		} else if ("title".equals(tagName)) {
		    setTitle(USmartDoc.getString(child));
		} else if ("subtitle".equals(tagName)) {
		    subtitle_ = USmartDoc.getString(child);
		} else if ("journal".equals(tagName)) {
		    journal_ = new Journal(child);
		    addContent(journal_);
		} else if ("pages".equals(tagName)) {
		    pages_ = USmartDoc.getString(child);
		} else if ("uri".equals(tagName)) {
		    uri_ = USmartDoc.getString(child);
		} else if ("a".equals(tagName)) { // XXX
		    uri_ = child.getAttribute("href");
		} else {
		    _warning("bad tag : " + tagName);
		}
		break;
	    case Node.TEXT_NODE:
	    case Node.ENTITY_REFERENCE_NODE:
	    case Node.COMMENT_NODE:
		// do nothing
		break;
	    default:
		throw (new InternalError("bad node type = " +
					 node.getNodeType())); // XXX : debug
	    }
	}
    }

    public String[] getAuthors() {
	String[] authors = new String[authors_.size()];
	return ((String[])authors_.toArray(authors));
    }

    public String getSubTitle() {
	return (subtitle_);
    }

    public String getPages() {
	return (pages_);
    }

    public String getUri() {
	return (uri_);
    }

    public Journal getJournal() {
	return (journal_);
    }
}
