/*
 * The JabaJaba class library
 *  Copyright (C) 1997-2000  ASAMI, Tomoharu (tasami@ibm.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package jp.gr.java_conf.jaba2.vfs;

/**
 * AbstractVFSObject
 *
 * @since   Dec. 13, 1998
 * @version Jan. 26, 2000
 * @author  ASAMI, Tomoharu (tasami@ibm.net)
 */
public abstract class AbstractVFSNode implements IVFSNode {
    private IVFSRealm realm_;
    private IVFSContainer parent_;
    private boolean isValid_;
    private String name_;

    protected AbstractVFSNode(String name) {
	parent_ = null;
	isValid_ = true;
	name_ = name;
    }

    public void setup(IFilter filter) {
	isValid_ = filter.isMatch(this);
    }

    public void setRealm(IVFSRealm realm) {
	realm_ = realm;
    }

    public void setParent(IVFSContainer parent) {
	parent_ = parent;
    }

    public final IVFSRealm getRealm() {
	return (realm_);
    }

    public final IVFSContainer getParent() {
	return (parent_);
    }

    public final String getName() {
	return (name_);
    }

    public String getPathname() {
	if (parent_ == null) {
	    return (name_);
	} else {
	    return (parent_.getPathname() + name_);
	}
    }

    public boolean isValid() {
	return (isValid_);
    }
}
